@@ -29,6 +29,12 @@ class S3:
 		if root is None:
 			root = ''
 		self._root = root
+		if spark is None or spark == True:
+			try:
+				from pyspark.sql.session import SparkSession
+				spark = SparkSession.builder.getOrCreate()
+			except ModuleNotFoundError:
+				pass
 		self._spark = spark
 		self._file_system = S3FileSystem(key=key, secret=secret, use_ssl=False)
 
@@ -199,15 +205,23 @@ class S3:
 
 	def read_bytes(self, path):
 		path = self._get_path(path=path)
-		with self.file_system.open(path=self._root+path, mode='rb') as f:
+		with self.file_system.open(path=path, mode='rb') as f:
 			result = f.read()
 		return result
 
 	def read_csv(self, path, header=True, encoding='utf-8', **kwargs):
 		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
-		with self.file_system.open(self._root+path, 'rb', ) as f:
-			df = pd.read_csv(f, header=header, encoding=encoding, **kwargs)
+
+		try:
+			with self.file_system.open(path, 'rb') as f:
+				# todo: header=True does not work, it gives an error, maybe use spark to read csv when you can
+				df = pd.read_csv(f, header=header, encoding=encoding, **kwargs)
+		except TypeError as exception:
+			if self.spark is not None and self.spark != True:
+				df = self.spark.read.csv(path, header=header)
+			else:
+				raise exception
 		return df
 
 	def write_pickle(self, obj, path, mode='overwite'):
@@ -496,12 +510,9 @@ class S3Path:
 
 		self._s3 = s3
 		self._path = path
-		self._name_and_extension = self._path.rstrip('/').split('/')[-1]
-		if '.' in self.name_and_extension:
-			self._name, self._extension = self.name_and_extension.rsplit('.')
-		else:
-			self._name = self.name_and_extension
-			self._extension = None
+		self._name_and_extension = s3.get_file_name_and_extension(path=path)
+		self._name = s3.get_file_name(path=path)
+		self._extension = s3.get_file_extension(path=path)
 
 	@property
 	def path(self):
